@@ -28,11 +28,22 @@ class Invalid(Exception):
     def __init__(self, error_dict):
         Exception.__init__(self,error_dict)
         self.error_dict=error_dict
+        m = []
+        for k,v in self.error_dict.items():
+            m.append( 'field "%s" %s'%(k,v.message))
+        self.message = '\n'.join(m)
+
 
     def __str__(self):
-        return self.error_dict.get('')
+        return self.message
     __unicode__ = __str__
 
+    # Hide Python 2.6 deprecation warnings.
+    def _get_message(self): return self._message
+    def _set_message(self, message): self._message = message
+    message = property(_get_message, _set_message)
+
+
 
 class Attribute(object):
     """
@@ -43,6 +54,7 @@ class Attribute(object):
     @ivar validator: Optional FormEncode validator.
     """
 
+    type = None
     title = None
     description = None
     validator = validatish.Always()
@@ -77,61 +89,71 @@ class Attribute(object):
         except validatish.Invalid, e:
             raise Invalid({'':e})
 
+    def __repr__(self):
+        attributes = []
+        if self.title:
+            attributes.append('title=%s'%repr(self.title))
+        if self.description:
+            attributes.append('description=%s'%repr(self.description))
+        if self.validator:
+            attributes.append('validator=%s'%repr(self.validator))
+        return 'schemaish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
+
 
 class String(Attribute):
     """
     A Python unicode instance.
     """
-    pass
+    type = 'String'
 
 
 class Integer(Attribute):
     """
     A Python integer.
     """
-    pass
+    type='Integer'
 
 
 class Float(Attribute):
     """
     A Python float.
     """
-    pass
+    type='Float'
 
 
 class Decimal(Attribute):
     """
     A decimal.Decimal instance.
     """
-    pass
+    type='Decimal'
 
 
 class Date(Attribute):
     """
     A datetime.date instance.
     """
-    pass
+    type='Date'
 
 
 class Time(Attribute):
     """
     A datetime.time instance.
     """
-    pass
+    type='Time'
 
 
 class DateTime(Attribute):
     """
     A datetime.datetime instance.
     """
-    pass
+    type='DateTime'
 
 
 class Boolean(Attribute):
     """
     A Python Boolean instance.
     """
-    pass
+    type='Boolean'
 
 
 class Sequence(Attribute):
@@ -140,7 +162,7 @@ class Sequence(Attribute):
 
     @ivar attr: Attribute type of items in the sequence.
     """
-
+    type = 'Sequence'
     attr = None
 
     def __init__(self, attr=None, **k):
@@ -177,7 +199,9 @@ class Sequence(Attribute):
             
         if error_dict:
             raise Invalid(error_dict)
-        
+
+    def __repr__(self):
+        return 'schemaish.Sequence(%r)'%self.attr
 
 class Tuple(Attribute):
     """
@@ -185,7 +209,8 @@ class Tuple(Attribute):
 
     @ivar attrs: List of Attributes that define the items in the tuple.
     """
-
+ 
+    type = 'Tuple'
     attrs = None
 
     def __init__(self, attrs=None, **k):
@@ -198,29 +223,45 @@ class Tuple(Attribute):
         if attrs is not None:
             self.attrs = attrs
 
+    def add(self, attr):
+        """
+        Add an attribute to the tuple
+
+        @param attr: Attribute type.
+        """
+        if attr is None:
+            self.attrs = [attr]
+        else:
+            self.attrs.append(attr)
+
     def validate(self, value):
         """
         Validate the tuple's items and the tuple itself.
         """
         if value:
             if len(self.attrs) != len(value):
-                raise Invalid({'':"Incorrect size"})
+                raise Invalid({'':validatish.Invalid("Incorrect size")})
             for attr, item in zip(self.attrs, value):
                 attr.validate(item)
         super(Tuple, self).validate(value)
 
+    def __repr__(self):
+        return 'schemaish.Tuple(%r)'%(self.attrs,)
+
 
 class _StructureMeta(type):
-
     def __init__(cls, name, bases, clsattrs):
+        # Gather attrs specific to this class.
+        cls.__schemaish_structure_attrs__ = list(
+            (name, value) for (name, value) in clsattrs.iteritems()
+            if isinstance(value, Attribute))
+        # Combine all attrs from this class and its subclasses.
         attrs = []
-        for (name, value) in clsattrs.items():
-            if isinstance(value, Attribute):
-                attrs.append((name, value))
-                del clsattrs[name]
-        attrs = [(a[1]._meta_order, a) for a in attrs]
-        attrs.sort()
-        attrs = [i[1] for i in attrs]
+        for c in cls.__mro__:
+            attrs.extend(getattr(c, '__schemaish_structure_attrs__', []))
+        # Sort the attrs to maintain the order as defined, and assign to the
+        # class.
+        attrs.sort(key=lambda i: i[1]._meta_order)
         cls.attrs = attrs
 
 
@@ -250,6 +291,7 @@ class Structure(Attribute):
         names and type of an attribute of the structure.
     """
 
+    type = 'Structure'
     __metaclass__ = _StructureMeta
 
     def __init__(self, attrs=None, **k):
@@ -314,11 +356,15 @@ class Structure(Attribute):
         if error_dict:
             raise Invalid(error_dict)
 
+    def __repr__(self):
+        item = '"%s": %s'
+        attrstrings = [item%a for a in self.attrs]
+        return 'schemaish.Structure(%s)'%(', '.join(attrstrings))
 
 
 class File(Attribute):
     """
     A File Object
     """
-    pass
+    type = 'File'
 
